@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,14 +27,10 @@
 #include <rocrand/rocrand.h>
 #include <rocrand/rocrand_kernel.h>
 #include <rocrand/rocrand_mtgp32_11213.h>
-#include <rocrand/rocrand_scrambled_sobol32_constants.h>
-#include <rocrand/rocrand_scrambled_sobol32_precomputed.h>
-#include <rocrand/rocrand_scrambled_sobol64_constants.h>
-#include <rocrand/rocrand_scrambled_sobol64_precomputed.h>
-#include <rocrand/rocrand_sobol32_precomputed.h>
-#include <rocrand/rocrand_sobol64_precomputed.h>
 
+#include "custom_csv_formater.hpp"
 #include <algorithm>
+#include <fstream>
 #include <iomanip>
 #include <iostream>
 #include <numeric>
@@ -46,8 +42,10 @@
 #endif
 
 template<typename EngineState>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_kernel(
-    EngineState* states, const unsigned long long seed, const unsigned long long offset)
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_kernel(EngineState*             states,
+                 const unsigned long long seed,
+                 const unsigned long long offset)
 {
     const unsigned int state_id = blockIdx.x * blockDim.x + threadIdx.x;
     EngineState        state;
@@ -56,8 +54,9 @@ __global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_kernel(
 }
 
 template<typename EngineState, typename T, typename Generator>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void generate_kernel(
-    EngineState* states, T* data, const size_t size, Generator generator)
+__global__
+__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_kernel(EngineState* states, T* data, const size_t size, Generator generator)
 {
     const unsigned int state_id = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int stride   = gridDim.x * blockDim.x;
@@ -84,7 +83,7 @@ struct runner
            const unsigned long long offset)
     {
         const size_t states_size = blocks * threads;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states), states_size * sizeof(EngineState)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(EngineState)));
 
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_kernel),
                            dim3(blocks),
@@ -125,12 +124,13 @@ struct runner
 };
 
 template<typename T, typename Generator>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void generate_kernel(
-    rocrand_state_mtgp32* states, T* data, const size_t size, Generator generator)
+__global__
+__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_kernel(rocrand_state_mtgp32* states, T* data, const size_t size, Generator generator)
 {
     const unsigned int state_id = blockIdx.x;
     unsigned int       index    = blockIdx.x * blockDim.x + threadIdx.x;
-    unsigned int       stride   = gridDim.x * blockDim.x;
+    unsigned int stride         = gridDim.x * blockDim.x;
 
     __shared__ rocrand_state_mtgp32 state;
     rocrand_mtgp32_block_copy(&states[state_id], &state);
@@ -166,8 +166,7 @@ struct runner<rocrand_state_mtgp32>
            const unsigned long long /* offset */)
     {
         const size_t states_size = std::min((size_t)200, blocks);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_mtgp32)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_mtgp32)));
 
         ROCRAND_CHECK(
             rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, states_size, seed));
@@ -198,8 +197,8 @@ struct runner<rocrand_state_mtgp32>
     }
 };
 
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_kernel(
-    rocrand_state_lfsr113* states, const uint4 seed)
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_kernel(rocrand_state_lfsr113* states, const uint4 seed)
 {
     const unsigned int    state_id = blockIdx.x * blockDim.x + threadIdx.x;
     rocrand_state_lfsr113 state;
@@ -219,8 +218,7 @@ struct runner<rocrand_state_lfsr113>
            const unsigned long long /* offset */)
     {
         const size_t states_size = blocks * threads;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_lfsr113)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_lfsr113)));
 
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_kernel),
                            dim3(blocks),
@@ -263,8 +261,9 @@ struct runner<rocrand_state_lfsr113>
 };
 
 template<typename EngineState, typename SobolType>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_sobol_kernel(
-    EngineState* states, SobolType* directions, SobolType offset)
+__global__
+__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_sobol_kernel(EngineState* states, SobolType* directions, SobolType offset)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id  = blockIdx.x * blockDim.x + threadIdx.x;
@@ -274,8 +273,12 @@ __global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_sobol_ker
 }
 
 template<typename EngineState, typename SobolType>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_scrambled_sobol_kernel(
-    EngineState* states, SobolType* directions, SobolType* scramble_constants, SobolType offset)
+__global__
+__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_scrambled_sobol_kernel(EngineState* states,
+                                 SobolType*   directions,
+                                 SobolType*   scramble_constants,
+                                 SobolType    offset)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id  = blockIdx.x * blockDim.x + threadIdx.x;
@@ -289,8 +292,9 @@ __global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void init_scrambled
 
 // generate_kernel for the normal and scrambled sobol generators
 template<typename EngineState, typename T, typename Generator>
-__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void generate_sobol_kernel(
-    EngineState* states, T* data, const size_t size, Generator generator)
+__global__
+__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_sobol_kernel(EngineState* states, T* data, const size_t size, Generator generator)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id  = blockIdx.x * blockDim.x + threadIdx.x;
@@ -324,17 +328,17 @@ struct runner<rocrand_state_sobol32>
     {
         this->dimensions = dimensions;
 
+        const unsigned int* h_directions;
+        ROCRAND_CHECK(
+            rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
         const size_t states_size = blocks * threads * dimensions;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_sobol32)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_sobol32)));
 
         unsigned int* directions;
         const size_t  size = dimensions * 32 * sizeof(unsigned int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&directions), size));
-        HIP_CHECK(hipMemcpy(directions,
-                            rocrand_h_sobol32_direction_vectors,
-                            size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&directions, size));
+        HIP_CHECK(hipMemcpy(directions, h_directions, size, hipMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_sobol_kernel),
@@ -392,25 +396,27 @@ struct runner<rocrand_state_scrambled_sobol32>
     {
         this->dimensions = dimensions;
 
+        const unsigned int* h_directions;
+        const unsigned int* h_constants;
+
+        ROCRAND_CHECK(
+            rocrand_get_direction_vectors32(&h_directions,
+                                            ROCRAND_SCRAMBLED_DIRECTION_VECTORS_32_JOEKUO6));
+        ROCRAND_CHECK(rocrand_get_scramble_constants32(&h_constants));
+
         const size_t states_size = blocks * threads * dimensions;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_scrambled_sobol32)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_scrambled_sobol32)));
 
         unsigned int* directions;
         const size_t  directions_size = dimensions * 32 * sizeof(unsigned int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&directions), directions_size));
-        HIP_CHECK(hipMemcpy(directions,
-                            rocrand_h_scrambled_sobol32_direction_vectors,
-                            directions_size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&directions, directions_size));
+        HIP_CHECK(hipMemcpy(directions, h_directions, directions_size, hipMemcpyHostToDevice));
 
         unsigned int* scramble_constants;
         const size_t  constants_size = dimensions * sizeof(unsigned int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&scramble_constants), constants_size));
-        HIP_CHECK(hipMemcpy(scramble_constants,
-                            h_scrambled_sobol32_constants,
-                            constants_size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&scramble_constants, constants_size));
+        HIP_CHECK(
+            hipMemcpy(scramble_constants, h_constants, constants_size, hipMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_scrambled_sobol_kernel),
@@ -470,17 +476,16 @@ struct runner<rocrand_state_sobol64>
     {
         this->dimensions = dimensions;
 
+        const unsigned long long* h_directions;
+        rocrand_get_direction_vectors64(&h_directions, ROCRAND_DIRECTION_VECTORS_64_JOEKUO6);
+
         const size_t states_size = blocks * threads * dimensions;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_sobol64)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_sobol64)));
 
         unsigned long long int* directions;
         const size_t            size = dimensions * 64 * sizeof(unsigned long long int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&directions), size));
-        HIP_CHECK(hipMemcpy(directions,
-                            rocrand_h_sobol64_direction_vectors,
-                            size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&directions, size));
+        HIP_CHECK(hipMemcpy(directions, h_directions, size, hipMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_sobol_kernel),
@@ -538,25 +543,26 @@ struct runner<rocrand_state_scrambled_sobol64>
     {
         this->dimensions = dimensions;
 
+        const unsigned long long* h_directions;
+        const unsigned long long* h_constants;
+
+        rocrand_get_direction_vectors64(&h_directions,
+                                        ROCRAND_SCRAMBLED_DIRECTION_VECTORS_64_JOEKUO6);
+        rocrand_get_scramble_constants64(&h_constants);
+
         const size_t states_size = blocks * threads * dimensions;
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&states),
-                            states_size * sizeof(rocrand_state_scrambled_sobol64)));
+        HIP_CHECK(hipMalloc(&states, states_size * sizeof(rocrand_state_scrambled_sobol64)));
 
         unsigned long long int* directions;
         const size_t            directions_size = dimensions * 64 * sizeof(unsigned long long int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&directions), directions_size));
-        HIP_CHECK(hipMemcpy(directions,
-                            rocrand_h_scrambled_sobol64_direction_vectors,
-                            directions_size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&directions, directions_size));
+        HIP_CHECK(hipMemcpy(directions, h_directions, directions_size, hipMemcpyHostToDevice));
 
         unsigned long long int* scramble_constants;
         const size_t            constants_size = dimensions * sizeof(unsigned long long int);
-        HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&scramble_constants), constants_size));
-        HIP_CHECK(hipMemcpy(scramble_constants,
-                            h_scrambled_sobol64_constants,
-                            constants_size,
-                            hipMemcpyHostToDevice));
+        HIP_CHECK(hipMalloc(&scramble_constants, constants_size));
+        HIP_CHECK(
+            hipMemcpy(scramble_constants, h_constants, constants_size, hipMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         hipLaunchKernelGGL(HIP_KERNEL_NAME(init_scrambled_sobol_kernel),
@@ -605,9 +611,9 @@ struct runner<rocrand_state_scrambled_sobol64>
 // Provide optional create and destroy functions for the generators.
 struct generator_type
 {
-    void create() {}
+    static void create() {}
 
-    void destroy() {}
+    static void destroy() {}
 };
 
 template<typename Engine>
@@ -620,7 +626,9 @@ struct generator_uint : public generator_type
         return "uniform-uint";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand(state);
     }
@@ -636,7 +644,9 @@ struct generator_ullong : public generator_type
         return "uniform-ullong";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand(state);
     }
@@ -652,7 +662,9 @@ struct generator_uniform : public generator_type
         return "uniform-float";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_uniform(state);
     }
@@ -668,7 +680,9 @@ struct generator_uniform_double : public generator_type
         return "uniform-double";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_uniform_double(state);
     }
@@ -684,7 +698,9 @@ struct generator_normal : public generator_type
         return "normal-float";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_normal(state);
     }
@@ -700,7 +716,9 @@ struct generator_normal_double : public generator_type
         return "normal-double";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_normal_double(state);
     }
@@ -716,7 +734,9 @@ struct generator_log_normal : public generator_type
         return "log-normal-float";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_log_normal(state, 0.f, 1.f);
     }
@@ -732,7 +752,9 @@ struct generator_log_normal_double : public generator_type
         return "log-normal-double";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state) const
     {
         return rocrand_log_normal_double(state, 0., 1.);
     }
@@ -750,7 +772,9 @@ struct generator_poisson : public generator_type
         return "poisson(lambda=" + stream.str() + ")";
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state)
     {
         return rocrand_poisson(state, lambda);
     }
@@ -780,7 +804,9 @@ struct generator_discrete_poisson : public generator_type
         ROCRAND_CHECK(rocrand_destroy_discrete_distribution(discrete_distribution));
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state)
     {
         return rocrand_discrete(state, discrete_distribution);
     }
@@ -820,7 +846,9 @@ struct generator_discrete_custom : public generator_type
         ROCRAND_CHECK(rocrand_destroy_discrete_distribution(discrete_distribution));
     }
 
-    __device__ data_type operator()(Engine* state)
+    __device__
+    data_type
+        operator()(Engine* state)
     {
         return rocrand_discrete(state, discrete_distribution);
     }
@@ -856,7 +884,7 @@ void run_benchmark(benchmark::State&        state,
     generator.create();
 
     data_type* data;
-    HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&data), size * sizeof(data_type)));
+    HIP_CHECK(hipMalloc(&data, size * sizeof(data_type)));
 
     constexpr unsigned long long int seed   = 12345ULL;
     constexpr unsigned long long int offset = 6789ULL;
@@ -905,14 +933,13 @@ template<typename Engine, typename Generator>
 void add_benchmark(const benchmark_context&                      context,
                    const hipStream_t                             stream,
                    std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                   const std::string&                            engine_name,
+                   const std::string&                            name,
                    Generator                                     generator)
 {
     static_assert(std::is_trivially_copyable<Generator>::value
                       && std::is_trivially_destructible<Generator>::value,
                   "Generator gets copied to device at kernel launch.");
-    const std::string benchmark_name
-        = "device_kernel<" + engine_name + "," + generator.name() + ">";
+    const std::string benchmark_name = "device_kernel<" + name + "," + generator.name() + ">";
     benchmarks.emplace_back(benchmark::RegisterBenchmark(benchmark_name.c_str(),
                                                          &run_benchmark<Engine, Generator>,
                                                          stream,
@@ -924,13 +951,15 @@ template<typename Engine>
 void add_benchmarks(const benchmark_context&                      ctx,
                     const hipStream_t                             stream,
                     std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                    const std::string&                            name)
+                    const rocrand_rng_type                        engine_type)
 {
     constexpr bool is_64_bits = std::is_same<Engine, rocrand_state_scrambled_sobol64>::value
                                 || std::is_same<Engine, rocrand_state_sobol64>::value
                                 || std::is_same<Engine, rocrand_state_threefry2x64_20>::value
                                 || std::is_same<Engine, rocrand_state_threefry4x64_20>::value;
 
+    const std::string name = engine_name(engine_type);
+
     if(is_64_bits)
     {
         add_benchmark<Engine>(ctx, stream, benchmarks, name, generator_ullong<Engine>());
@@ -966,6 +995,14 @@ void add_benchmarks(const benchmark_context&                      ctx,
 
 int main(int argc, char* argv[])
 {
+    // get paramaters before they are passed into
+    // benchmark::Initialize()
+    std::string outFormat     = "";
+    std::string filter        = "";
+    std::string consoleFormat = "";
+
+    getFormats(argc, argv, outFormat, filter, consoleFormat);
+
     benchmark::Initialize(&argc, argv);
 
     cli::Parser parser(argc, argv);
@@ -1007,20 +1044,41 @@ int main(int argc, char* argv[])
     std::vector<benchmark::internal::Benchmark*> benchmarks = {};
 
     // MT19937 has no kernel implementation
-    add_benchmarks<rocrand_state_lfsr113>(ctx, stream, benchmarks, "lfsr113");
-    add_benchmarks<rocrand_state_mrg31k3p>(ctx, stream, benchmarks, "mrg31k3p");
-    add_benchmarks<rocrand_state_mrg32k3a>(ctx, stream, benchmarks, "mrg32k3a");
-    add_benchmarks<rocrand_state_mtgp32>(ctx, stream, benchmarks, "mtgp32");
-    add_benchmarks<rocrand_state_philox4x32_10>(ctx, stream, benchmarks, "philox4x32_10");
-    add_benchmarks<rocrand_state_scrambled_sobol32>(ctx, stream, benchmarks, "scrambled_sobol32");
-    add_benchmarks<rocrand_state_scrambled_sobol64>(ctx, stream, benchmarks, "scrambled_sobol64");
-    add_benchmarks<rocrand_state_sobol32>(ctx, stream, benchmarks, "sobol32");
-    add_benchmarks<rocrand_state_sobol64>(ctx, stream, benchmarks, "sobol64");
-    add_benchmarks<rocrand_state_threefry2x32_20>(ctx, stream, benchmarks, "threefry2x32_20");
-    add_benchmarks<rocrand_state_threefry4x32_20>(ctx, stream, benchmarks, "threefry4x32_20");
-    add_benchmarks<rocrand_state_threefry2x64_20>(ctx, stream, benchmarks, "threefry2x64_20");
-    add_benchmarks<rocrand_state_threefry4x64_20>(ctx, stream, benchmarks, "threefry4x64_20");
-    add_benchmarks<rocrand_state_xorwow>(ctx, stream, benchmarks, "xorwow");
+    add_benchmarks<rocrand_state_lfsr113>(ctx, stream, benchmarks, ROCRAND_RNG_PSEUDO_LFSR113);
+    add_benchmarks<rocrand_state_mrg31k3p>(ctx, stream, benchmarks, ROCRAND_RNG_PSEUDO_MRG31K3P);
+    add_benchmarks<rocrand_state_mrg32k3a>(ctx, stream, benchmarks, ROCRAND_RNG_PSEUDO_MRG32K3A);
+    add_benchmarks<rocrand_state_mtgp32>(ctx, stream, benchmarks, ROCRAND_RNG_PSEUDO_MTGP32);
+    add_benchmarks<rocrand_state_philox4x32_10>(ctx,
+                                                stream,
+                                                benchmarks,
+                                                ROCRAND_RNG_PSEUDO_PHILOX4_32_10);
+    add_benchmarks<rocrand_state_scrambled_sobol32>(ctx,
+                                                    stream,
+                                                    benchmarks,
+                                                    ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL32);
+    add_benchmarks<rocrand_state_scrambled_sobol64>(ctx,
+                                                    stream,
+                                                    benchmarks,
+                                                    ROCRAND_RNG_QUASI_SCRAMBLED_SOBOL64);
+    add_benchmarks<rocrand_state_sobol32>(ctx, stream, benchmarks, ROCRAND_RNG_QUASI_SOBOL32);
+    add_benchmarks<rocrand_state_sobol64>(ctx, stream, benchmarks, ROCRAND_RNG_QUASI_SOBOL64);
+    add_benchmarks<rocrand_state_threefry2x32_20>(ctx,
+                                                  stream,
+                                                  benchmarks,
+                                                  ROCRAND_RNG_PSEUDO_THREEFRY2_32_20);
+    add_benchmarks<rocrand_state_threefry4x32_20>(ctx,
+                                                  stream,
+                                                  benchmarks,
+                                                  ROCRAND_RNG_PSEUDO_THREEFRY4_32_20);
+    add_benchmarks<rocrand_state_threefry2x64_20>(ctx,
+                                                  stream,
+                                                  benchmarks,
+                                                  ROCRAND_RNG_PSEUDO_THREEFRY2_64_20);
+    add_benchmarks<rocrand_state_threefry4x64_20>(ctx,
+                                                  stream,
+                                                  benchmarks,
+                                                  ROCRAND_RNG_PSEUDO_THREEFRY4_64_20);
+    add_benchmarks<rocrand_state_xorwow>(ctx, stream, benchmarks, ROCRAND_RNG_PSEUDO_XORWOW);
 
     // Use manual timing
     for(auto& b : benchmarks)
@@ -1029,8 +1087,16 @@ int main(int argc, char* argv[])
         b->Unit(benchmark::kMillisecond);
     }
 
+    benchmark::BenchmarkReporter* console_reporter  = getConsoleReporter(consoleFormat);
+    benchmark::BenchmarkReporter* out_file_reporter = getOutFileReporter(outFormat);
+
+    std::string spec = (filter == "" || filter == "all") ? "." : filter;
+
     // Run benchmarks
-    benchmark::RunSpecifiedBenchmarks();
+    if(outFormat == "") // default case
+        benchmark::RunSpecifiedBenchmarks(console_reporter, spec);
+    else
+        benchmark::RunSpecifiedBenchmarks(console_reporter, out_file_reporter, spec);
     HIP_CHECK(hipStreamDestroy(stream));
 
     return 0;
