@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -22,29 +22,71 @@
 #define ROCRAND_COMMON_H_
 
 #define ROCRAND_2POW16_INV (1.5258789e-05f)
-#define ROCRAND_2POW16_INV_2PI (1.5258789e-05f * 6.2831855f)
+#define ROCRAND_2POW16_INV_2PI (9.58738e-05f)
 #define ROCRAND_2POW32_INV (2.3283064e-10f)
 #define ROCRAND_2POW32_INV_DOUBLE (2.3283064365386963e-10)
 #define ROCRAND_2POW64_INV (5.4210109e-20f)
 #define ROCRAND_2POW64_INV_DOUBLE (5.4210108624275221700372640043497e-20)
-#define ROCRAND_2POW32_INV_2PI (2.3283064e-10f * 6.2831855f)
+#define ROCRAND_2POW32_INV_2PI (1.46291807e-09f)
 #define ROCRAND_2POW53_INV_DOUBLE (1.1102230246251565e-16)
-#define ROCRAND_PI  (3.1415926f)
+#define ROCRAND_PI (3.141592653f)
 #define ROCRAND_PI_DOUBLE  (3.1415926535897932)
 #define ROCRAND_2PI (6.2831855f)
 #define ROCRAND_SQRT2 (1.4142135f)
 #define ROCRAND_SQRT2_DOUBLE (1.4142135623730951)
 
+#include <hip/hip_runtime.h>
+
 #include <math.h>
 
 #define ROCRAND_KERNEL __global__ static
 
-#ifndef FQUALIFIERS
-#define FQUALIFIERS __forceinline__ __device__
-#endif // FQUALIFIERS
+#if __HIP_DEVICE_COMPILE__            \
+    && (defined(__HIP_PLATFORM_AMD__) \
+        || (defined(__HIP_PLATFORM_NVCC__) && (__CUDA_ARCH__ >= 530)))
+    #define ROCRAND_HALF_MATH_SUPPORTED
+#endif
 
-#if __HIP_DEVICE_COMPILE__ && (defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__) || (defined(__HIP_PLATFORM_NVCC__) && (__CUDA_ARCH__ >= 530)))
-#define ROCRAND_HALF_MATH_SUPPORTED
+//  Copyright 2001 John Maddock.
+//  Copyright 2017 Peter Dimov.
+//
+//  Distributed under the Boost Software License, Version 1.0.
+//
+//  See http://www.boost.org/LICENSE_1_0.txt
+//
+//  BOOST_STRINGIZE(X)
+#define ROCRAND_STRINGIZE(X) ROCRAND_DO_STRINGIZE(X)
+#define ROCRAND_DO_STRINGIZE(X) #X
+
+//  Copyright 2017 Peter Dimov.
+//
+//  Distributed under the Boost Software License, Version 1.0.
+//
+//  See http://www.boost.org/LICENSE_1_0.txt
+//
+//  BOOST_PRAGMA_MESSAGE("message")
+//
+//  Expands to the equivalent of #pragma message("message")
+#if defined(__INTEL_COMPILER)
+    #define ROCRAND_PRAGMA_MESSAGE(x) \
+        __pragma(message(__FILE__ "(" ROCRAND_STRINGIZE(__LINE__) "): note: " x))
+#elif defined(__GNUC__)
+    #define ROCRAND_PRAGMA_MESSAGE(x) _Pragma(ROCRAND_STRINGIZE(message(x)))
+#elif defined(_MSC_VER)
+    #define ROCRAND_PRAGMA_MESSAGE(x) \
+        __pragma(message(__FILE__ "(" ROCRAND_STRINGIZE(__LINE__) "): note: " x))
+#else
+    #define ROCRAND_PRAGMA_MESSAGE(x)
+#endif
+
+#if __cplusplus >= 201402L
+    #define ROCRAND_DEPRECATED(msg) [[deprecated(msg)]]
+#elif defined(_MSC_VER) && !defined(__clang__)
+    #define ROCRAND_DEPRECATED(msg) __declspec(deprecated(msg))
+#elif defined(__clang__) || defined(__GNUC__)
+    #define ROCRAND_DEPRECATED(msg) __attribute__((deprecated(msg)))
+#else
+    #define ROCRAND_DEPRECATED(msg)
 #endif
 
 namespace rocrand_device {
@@ -71,13 +113,13 @@ namespace detail {
   #endif
 #endif
 
-FQUALIFIERS
-unsigned long long mad_u64_u32(const unsigned int x, const unsigned int y, const unsigned long long z)
+__forceinline__ __device__ __host__ unsigned long long
+    mad_u64_u32(const unsigned int x, const unsigned int y, const unsigned long long z)
 {
-  #if defined(__HIP_PLATFORM_HCC__) && defined(__HIP_DEVICE_COMPILE__) \
+#if defined(__HIP_PLATFORM_AMD__) && defined(__HIP_DEVICE_COMPILE__) \
     && defined(ROCRAND_ENABLE_INLINE_ASM)
 
-  #if __AMDGCN_WAVEFRONT_SIZE == 64u
+    #if __AMDGCN_WAVEFRONT_SIZE == 64u
     using sgpr_t = unsigned long long;
   #elif __AMDGCN_WAVEFRONT_SIZE == 32u
     using sgpr_t = unsigned int;
@@ -114,41 +156,35 @@ unsigned long long mad_u64_u32(const unsigned int x, const unsigned int y, const
 template<typename Engine>
 struct engine_boxmuller_helper
 {
-    static FQUALIFIERS
-    bool has_float(const Engine * engine)
+    static __forceinline__ __device__ __host__ bool has_float(const Engine* engine)
     {
         return engine->m_state.boxmuller_float_state != 0;
     }
 
-    static FQUALIFIERS
-    float get_float(Engine * engine)
+    static __forceinline__ __device__ __host__ float get_float(Engine* engine)
     {
         engine->m_state.boxmuller_float_state = 0;
         return engine->m_state.boxmuller_float;
     }
 
-    static FQUALIFIERS
-    void save_float(Engine * engine, float f)
+    static __forceinline__ __device__ __host__ void save_float(Engine* engine, float f)
     {
         engine->m_state.boxmuller_float_state = 1;
         engine->m_state.boxmuller_float = f;
     }
 
-    static FQUALIFIERS
-    bool has_double(const Engine * engine)
+    static __forceinline__ __device__ __host__ bool has_double(const Engine* engine)
     {
         return engine->m_state.boxmuller_double_state != 0;
     }
 
-    static FQUALIFIERS
-    float get_double(Engine * engine)
+    static __forceinline__ __device__ __host__ float get_double(Engine* engine)
     {
         engine->m_state.boxmuller_double_state = 0;
         return engine->m_state.boxmuller_double;
     }
 
-    static FQUALIFIERS
-    void save_double(Engine * engine, double d)
+    static __forceinline__ __device__ __host__ void save_double(Engine* engine, double d)
     {
         engine->m_state.boxmuller_double_state = 1;
         engine->m_state.boxmuller_double = d;
@@ -156,17 +192,18 @@ struct engine_boxmuller_helper
 };
 
 template<typename T>
-FQUALIFIERS void split_ull(T& lo, T& hi, unsigned long long int val);
+__forceinline__ __device__ __host__ void split_ull(T& lo, T& hi, unsigned long long int val);
 
 template<>
-FQUALIFIERS void split_ull(unsigned int& lo, unsigned int& hi, unsigned long long int val)
+__forceinline__ __device__ __host__ void
+    split_ull(unsigned int& lo, unsigned int& hi, unsigned long long int val)
 {
     lo = val & 0xFFFFFFFF;
     hi = (val >> 32) & 0xFFFFFFFF;
 }
 
 template<>
-FQUALIFIERS void
+__forceinline__ __device__ __host__ void
     split_ull(unsigned long long int& lo, unsigned long long int& hi, unsigned long long int val)
 {
     lo = val;
