@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,10 +21,6 @@
 #ifndef ROCRAND_DISCRETE_H_
 #define ROCRAND_DISCRETE_H_
 
-#ifndef FQUALIFIERS
-#define FQUALIFIERS __forceinline__ __device__
-#endif // FQUALIFIERS
-
 #include <math.h>
 
 #include "rocrand/rocrand_lfsr113.h"
@@ -43,8 +39,13 @@
 #include "rocrand/rocrand_xorwow.h"
 
 #include "rocrand/rocrand_discrete_types.h"
-#include "rocrand/rocrand_normal.h"
-#include "rocrand/rocrand_uniform.h"
+
+// On certain architectures such as NAVI2 and NAVI3, double arithmetic is significantly slower.
+// In such cases we want to prefer the CDF method over the alias method.
+// This macro is undefined at the end of the file.
+#if defined(__HIP_DEVICE_COMPILE__) && (defined(__GFX10__) || defined(__GFX11__))
+    #define ROCRAND_PREFER_CDF_OVER_ALIAS
+#endif
 
 // Alias method
 //
@@ -57,11 +58,12 @@
 namespace rocrand_device {
 namespace detail {
 
-FQUALIFIERS unsigned int discrete_alias(const double       x,
-                                        const unsigned int size,
-                                        const unsigned int offset,
-                                        const unsigned int* __restrict__ alias,
-                                        const double* __restrict__ probability)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const double       x,
+                   const unsigned int size,
+                   const unsigned int offset,
+                   const unsigned int* __restrict__ alias,
+                   const double* __restrict__ probability)
 {
     // Calculate value using Alias table
 
@@ -73,13 +75,14 @@ FQUALIFIERS unsigned int discrete_alias(const double       x,
     return offset + (y < probability[i] ? i : alias[i]);
 }
 
-FQUALIFIERS unsigned int discrete_alias(const double x, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const double x, const rocrand_discrete_distribution_st& dis)
 {
     return discrete_alias(x, dis.size, dis.offset, dis.alias, dis.probability);
 }
 
-FQUALIFIERS
-unsigned int discrete_alias(const unsigned int r, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
@@ -87,26 +90,26 @@ unsigned int discrete_alias(const unsigned int r, const rocrand_discrete_distrib
 }
 
 // To prevent ambiguity compile error when compiler is facing the type "unsigned long"!!!
-FQUALIFIERS unsigned int discrete_alias(const unsigned long                     r,
-                                        const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned long r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
     return discrete_alias(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_alias(const unsigned long long int            r,
-                                        const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned long long int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_64 = ROCRAND_2POW64_INV_DOUBLE;
     const double x = r * inv_double_64;
     return discrete_alias(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_cdf(const double       x,
-                                      const unsigned int size,
-                                      const unsigned int offset,
-                                      const double* __restrict__ cdf)
+__forceinline__ __device__ __host__ unsigned int discrete_cdf(const double       x,
+                                                              const unsigned int size,
+                                                              const unsigned int offset,
+                                                              const double* __restrict__ cdf)
 {
     // Calculate value using binary search in CDF
 
@@ -130,13 +133,14 @@ FQUALIFIERS unsigned int discrete_cdf(const double       x,
     return offset + min;
 }
 
-FQUALIFIERS unsigned int discrete_cdf(const double x, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const double x, const rocrand_discrete_distribution_st& dis)
 {
     return discrete_cdf(x, dis.size, dis.offset, dis.cdf);
 }
 
-FQUALIFIERS
-unsigned int discrete_cdf(const unsigned int r, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
@@ -144,16 +148,16 @@ unsigned int discrete_cdf(const unsigned int r, const rocrand_discrete_distribut
 }
 
 // To prevent ambiguity compile error when compiler is facing the type "unsigned long"!!!
-FQUALIFIERS unsigned int discrete_cdf(const unsigned long                     r,
-                                      const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned long r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
     return discrete_cdf(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_cdf(const unsigned long long int            r,
-                                      const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned long long int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_64 = ROCRAND_2POW64_INV_DOUBLE;
     const double x = r * inv_double_64;
@@ -180,8 +184,9 @@ FQUALIFIERS unsigned int discrete_cdf(const unsigned long long int            r,
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_philox4x32_10 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_philox4x32_10*        state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -198,8 +203,8 @@ unsigned int rocrand_discrete(rocrand_state_philox4x32_10 * state, const rocrand
  *
  * \return Four <tt>unsigned int</tt> values distributed according to \p discrete_distribution as \p uint4
  */
-FQUALIFIERS
-uint4 rocrand_discrete4(rocrand_state_philox4x32_10 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ uint4 rocrand_discrete4(
+    rocrand_state_philox4x32_10* state, const rocrand_discrete_distribution discrete_distribution)
 {
     const uint4 u4 = rocrand4(state);
     return uint4 {
@@ -222,8 +227,9 @@ uint4 rocrand_discrete4(rocrand_state_philox4x32_10 * state, const rocrand_discr
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_mrg31k3p*             state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_mrg31k3p*             state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -240,8 +246,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_mrg31k3p*             st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_mrg32k3a * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_mrg32k3a*             state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -258,8 +265,9 @@ unsigned int rocrand_discrete(rocrand_state_mrg32k3a * state, const rocrand_disc
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_xorwow * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_xorwow*               state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -276,10 +284,15 @@ unsigned int rocrand_discrete(rocrand_state_xorwow * state, const rocrand_discre
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_mtgp32 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ unsigned int
+    rocrand_discrete(rocrand_state_mtgp32*               state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /**
@@ -294,8 +307,9 @@ unsigned int rocrand_discrete(rocrand_state_mtgp32 * state, const rocrand_discre
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_sobol32 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_sobol32*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -312,9 +326,9 @@ unsigned int rocrand_discrete(rocrand_state_sobol32 * state, const rocrand_discr
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
-                              const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -331,8 +345,9 @@ unsigned int rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
  *
  * \return <tt>unsigned long long int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_sobol64*              state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_sobol64*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -349,8 +364,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_sobol64*              st
  *
  * \return <tt>unsigned long long int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_scrambled_sobol64*    state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_scrambled_sobol64*    state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -367,11 +383,15 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_scrambled_sobol64*    st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_lfsr113*              state,
-                              const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_lfsr113*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /**
@@ -386,10 +406,15 @@ unsigned int rocrand_discrete(rocrand_state_lfsr113*              state,
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x32_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry2x32_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /**
@@ -404,10 +429,15 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x32_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x64_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry2x64_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /**
@@ -422,10 +452,15 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x64_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x32_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry4x32_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /**
@@ -440,12 +475,22 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x32_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x64_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry4x64_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
+#ifdef ROCRAND_PREFER_CDF_OVER_ALIAS
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
+#else
+    return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
+#endif
 }
 
 /** @} */ // end of group rocranddevice
 
+// Undefine the macro that may be defined at the top of the file!
+#if defined(ROCRAND_PREFER_CDF_OVER_ALIAS)
+    #undef ROCRAND_PREFER_CDF_OVER_ALIAS
+#endif
+
 #endif // ROCRAND_DISCRETE_H_
