@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,10 +21,6 @@
 #ifndef ROCRAND_LOG_NORMAL_H_
 #define ROCRAND_LOG_NORMAL_H_
 
-#ifndef FQUALIFIERS
-#define FQUALIFIERS __forceinline__ __device__
-#endif // FQUALIFIERS
-
 /** \rocrand_internal \addtogroup rocranddevice
  *
  *  @{
@@ -64,9 +60,9 @@
  *
  * \return Log-normally distributed \p float value
  */
-#ifndef ROCRAND_DETAIL_PHILOX_BM_NOT_IN_STATE
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_philox4x32_10 * state, float mean, float stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_philox4x32_10* state, float mean, float stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_philox4x32_10> bm_helper;
 
@@ -82,7 +78,7 @@ float rocrand_log_normal(rocrand_state_philox4x32_10 * state, float mean, float
     bm_helper::save_float(state, r.y);
     return expf(mean + (stddev * r.x));
 }
-#endif // ROCRAND_DETAIL_PHILOX_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p float values.
@@ -98,8 +94,9 @@ float rocrand_log_normal(rocrand_state_philox4x32_10 * state, float mean, float
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS
-float2 rocrand_log_normal2(rocrand_state_philox4x32_10 * state, float mean, float stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_philox4x32_10* state,
+                                                               float                        mean,
+                                                               float                        stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -125,8 +122,9 @@ float2 rocrand_log_normal2(rocrand_state_philox4x32_10 * state, float mean, floa
  *
  * \return Four log-normally distributed \p float value as \p float4
  */
-FQUALIFIERS
-float4 rocrand_log_normal4(rocrand_state_philox4x32_10 * state, float mean, float stddev)
+__forceinline__ __device__ __host__ float4 rocrand_log_normal4(rocrand_state_philox4x32_10* state,
+                                                               float                        mean,
+                                                               float                        stddev)
 {
     float4 r = rocrand_device::detail::normal_distribution4(rocrand4(state));
     return float4 {
@@ -152,9 +150,9 @@ float4 rocrand_log_normal4(rocrand_state_philox4x32_10 * state, float mean, floa
  *
  * \return Log-normally distributed \p double value
  */
-#ifndef ROCRAND_DETAIL_PHILOX_BM_NOT_IN_STATE
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_philox4x32_10 * state, double mean, double stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_philox4x32_10* state, double mean, double stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_philox4x32_10> bm_helper;
 
@@ -166,7 +164,7 @@ double rocrand_log_normal_double(rocrand_state_philox4x32_10 * state, double mea
     bm_helper::save_double(state, r.y);
     return exp(mean + r.x * stddev);
 }
-#endif // ROCRAND_DETAIL_PHILOX_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p double values.
@@ -182,8 +180,8 @@ double rocrand_log_normal_double(rocrand_state_philox4x32_10 * state, double mea
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS
-double2 rocrand_log_normal_double2(rocrand_state_philox4x32_10 * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_philox4x32_10* state, double mean, double stddev)
 {
     double2 r = rocrand_device::detail::normal_distribution_double2(rocrand4(state));
     return double2 {
@@ -206,8 +204,8 @@ double2 rocrand_log_normal_double2(rocrand_state_philox4x32_10 * state, double m
  *
  * \return Four log-normally distributed \p double values as \p double4
  */
-FQUALIFIERS
-double4 rocrand_log_normal_double4(rocrand_state_philox4x32_10 * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double4
+    rocrand_log_normal_double4(rocrand_state_philox4x32_10* state, double mean, double stddev)
 {
     double2 r1, r2;
     r1 = rocrand_log_normal_double2(state, mean, stddev);
@@ -232,8 +230,9 @@ double4 rocrand_log_normal_double4(rocrand_state_philox4x32_10 * state, double m
  *
  * \return Log-normally distributed \p float value
  */
-#ifndef ROCRAND_DETAIL_MRG31K3P_BM_NOT_IN_STATE
-FQUALIFIERS float rocrand_log_normal(rocrand_state_mrg31k3p* state, float mean, float stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_mrg31k3p* state, float mean, float stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_mrg31k3p> bm_helper;
 
@@ -250,7 +249,7 @@ FQUALIFIERS float rocrand_log_normal(rocrand_state_mrg31k3p* state, float mean,
     bm_helper::save_float(state, r.y);
     return expf(mean + (stddev * r.x));
 }
-#endif // ROCRAND_DETAIL_MRG31K3P_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p float values.
@@ -266,7 +265,9 @@ FQUALIFIERS float rocrand_log_normal(rocrand_state_mrg31k3p* state, float mean,
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_mrg31k3p* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_mrg31k3p* state,
+                                                               float                   mean,
+                                                               float                   stddev)
 {
     auto state1 = state->next();
     auto state2 = state->next();
@@ -291,8 +292,8 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_mrg31k3p* state, float mean
  *
  * \return Log-normally distributed \p double value
  */
-#ifndef ROCRAND_DETAIL_MRG31K3P_BM_NOT_IN_STATE
-FQUALIFIERS double
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ double
     rocrand_log_normal_double(rocrand_state_mrg31k3p* state, double mean, double stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_mrg31k3p> bm_helper;
@@ -311,7 +312,7 @@ FQUALIFIERS double
     bm_helper::save_double(state, r.y);
     return exp(mean + r.x * stddev);
 }
-#endif // ROCRAND_DETAIL_MRG31K3P_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p double values.
@@ -327,9 +328,8 @@ FQUALIFIERS double
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_mrg31k3p* state,
-                                               double                  mean,
-                                               double                  stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_mrg31k3p* state, double mean, double stddev)
 {
     auto state1 = state->next();
     auto state2 = state->next();
@@ -355,9 +355,9 @@ FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_mrg31k3p* state,
  *
  * \return Log-normally distributed \p float value
  */
-#ifndef ROCRAND_DETAIL_MRG32K3A_BM_NOT_IN_STATE
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_mrg32k3a * state, float mean, float stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_mrg32k3a* state, float mean, float stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_mrg32k3a> bm_helper;
 
@@ -374,7 +374,7 @@ float rocrand_log_normal(rocrand_state_mrg32k3a * state, float mean, float stdde
     bm_helper::save_float(state, r.y);
     return expf(mean + (stddev * r.x));
 }
-#endif // ROCRAND_DETAIL_MRG32K3A_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p float values.
@@ -390,8 +390,9 @@ float rocrand_log_normal(rocrand_state_mrg32k3a * state, float mean, float stdde
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS
-float2 rocrand_log_normal2(rocrand_state_mrg32k3a * state, float mean, float stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_mrg32k3a* state,
+                                                               float                   mean,
+                                                               float                   stddev)
 {
     auto state1 = state->next();
     auto state2 = state->next();
@@ -419,9 +420,9 @@ float2 rocrand_log_normal2(rocrand_state_mrg32k3a * state, float mean, float std
  *
  * \return Log-normally distributed \p double value
  */
-#ifndef ROCRAND_DETAIL_MRG32K3A_BM_NOT_IN_STATE
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_mrg32k3a * state, double mean, double stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_mrg32k3a* state, double mean, double stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_mrg32k3a> bm_helper;
 
@@ -439,7 +440,7 @@ double rocrand_log_normal_double(rocrand_state_mrg32k3a * state, double mean, do
     bm_helper::save_double(state, r.y);
     return exp(mean + r.x * stddev);
 }
-#endif // ROCRAND_DETAIL_MRG32K3A_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p double values.
@@ -455,8 +456,8 @@ double rocrand_log_normal_double(rocrand_state_mrg32k3a * state, double mean, do
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS
-double2 rocrand_log_normal_double2(rocrand_state_mrg32k3a * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_mrg32k3a* state, double mean, double stddev)
 {
     auto state1 = state->next();
     auto state2 = state->next();
@@ -485,9 +486,9 @@ double2 rocrand_log_normal_double2(rocrand_state_mrg32k3a * state, double mean,
  *
  * \return Log-normally distributed \p float value
  */
-#ifndef ROCRAND_DETAIL_XORWOW_BM_NOT_IN_STATE
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_xorwow * state, float mean, float stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_xorwow* state, float mean, float stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_xorwow> bm_helper;
 
@@ -503,7 +504,7 @@ float rocrand_log_normal(rocrand_state_xorwow * state, float mean, float stddev)
     bm_helper::save_float(state, r.y);
     return expf(mean + (stddev * r.x));
 }
-#endif // ROCRAND_DETAIL_XORWOW_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p float values.
@@ -519,8 +520,9 @@ float rocrand_log_normal(rocrand_state_xorwow * state, float mean, float stddev)
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS
-float2 rocrand_log_normal2(rocrand_state_xorwow * state, float mean, float stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_xorwow* state,
+                                                               float                 mean,
+                                                               float                 stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -547,9 +549,9 @@ float2 rocrand_log_normal2(rocrand_state_xorwow * state, float mean, float stdde
  *
  * \return Log-normally distributed \p double value
  */
-#ifndef ROCRAND_DETAIL_XORWOW_BM_NOT_IN_STATE
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_xorwow * state, double mean, double stddev)
+#ifndef ROCRAND_DETAIL_BM_NOT_IN_STATE
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_xorwow* state, double mean, double stddev)
 {
     typedef rocrand_device::detail::engine_boxmuller_helper<rocrand_state_xorwow> bm_helper;
 
@@ -569,7 +571,7 @@ double rocrand_log_normal_double(rocrand_state_xorwow * state, double mean, doub
     bm_helper::save_double(state, r.y);
     return exp(mean + (stddev * r.x));
 }
-#endif // ROCRAND_DETAIL_XORWOW_BM_NOT_IN_STATE
+#endif // ROCRAND_DETAIL_BM_NOT_IN_STATE
 
 /**
  * \brief Returns two log-normally distributed \p double values.
@@ -585,8 +587,9 @@ double rocrand_log_normal_double(rocrand_state_xorwow * state, double mean, doub
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS
-double2 rocrand_log_normal_double2(rocrand_state_xorwow * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double2 rocrand_log_normal_double2(rocrand_state_xorwow* state,
+                                                                       double                mean,
+                                                                       double                stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -614,13 +617,38 @@ double2 rocrand_log_normal_double2(rocrand_state_xorwow * state, double mean, do
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_mtgp32 * state, float mean, float stddev)
+__forceinline__ __device__ float
+    rocrand_log_normal(rocrand_state_mtgp32* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
 }
 
+/**
+ * \brief Returns two log-normally distributed \p float values.
+ *
+ * Generates and returns two log-normally distributed \p float values using MTGP32
+ * generator in \p state, and increments position of the generator by two.
+ * The function uses the Box-Muller transform method to generate two normally distributed
+ * values, transforms them to log-normally distributed values, and returns both.
+ *
+ * \param state  - Pointer to a state to use
+ * \param mean   - Mean of the related log-normal distribution
+ * \param stddev - Standard deviation of the related log-normal distribution
+ *
+ * \return Two log-normally distributed \p float value as \p float2
+ */
+__forceinline__ __device__ float2 rocrand_log_normal2(rocrand_state_mtgp32* state,
+                                                      float                 mean,
+                                                      float                 stddev)
+{
+    auto state1 = rocrand(state);
+    auto state2 = rocrand(state);
+
+    float2 r = rocrand_device::detail::normal_distribution2(state1, state2);
+    return float2{expf(mean + (stddev * r.x)), expf(mean + (stddev * r.y))};
+}
+
 /**
  * \brief Returns a log-normally distributed \p double value.
  *
@@ -633,13 +661,41 @@ float rocrand_log_normal(rocrand_state_mtgp32 * state, float mean, float stddev)
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_mtgp32 * state, double mean, double stddev)
+__forceinline__ __device__ double
+    rocrand_log_normal_double(rocrand_state_mtgp32* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
 }
 
+/**
+ * \brief Returns two log-normally distributed \p double values.
+ *
+ * Generates and returns two log-normally distributed \p double values using MTGP32
+ * generator in \p state, and increments position of the generator by four.
+ * The function uses the Box-Muller transform method to generate two normally distributed
+ * values, transforms them to log-normally distributed values, and returns both.
+ *
+ * \param state  - Pointer to a state to use
+ * \param mean   - Mean of the related log-normal distribution
+ * \param stddev - Standard deviation of the related log-normal distribution
+ *
+ * \return Two log-normally distributed \p double values as \p double2
+ */
+__forceinline__ __device__ double2 rocrand_log_normal_double2(rocrand_state_mtgp32* state,
+                                                              double                mean,
+                                                              double                stddev)
+{
+    auto state1 = rocrand(state);
+    auto state2 = rocrand(state);
+    auto state3 = rocrand(state);
+    auto state4 = rocrand(state);
+
+    double2 r = rocrand_device::detail::normal_distribution_double2(
+        uint4{state1, state2, state3, state4});
+    return double2{exp(mean + (stddev * r.x)), exp(mean + (stddev * r.y))};
+}
+
 /**
  * \brief Returns a log-normally distributed \p float value.
  *
@@ -652,8 +708,8 @@ double rocrand_log_normal_double(rocrand_state_mtgp32 * state, double mean, doub
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_sobol32 * state, float mean, float stddev)
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_sobol32* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
@@ -671,8 +727,8 @@ float rocrand_log_normal(rocrand_state_sobol32 * state, float mean, float stddev
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_sobol32 * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_sobol32* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
@@ -690,8 +746,8 @@ double rocrand_log_normal_double(rocrand_state_sobol32 * state, double mean, dou
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_scrambled_sobol32* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_scrambled_sobol32* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
@@ -709,8 +765,8 @@ float rocrand_log_normal(rocrand_state_scrambled_sobol32* state, float mean, flo
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_scrambled_sobol32* state, double mean, double stddev)
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_scrambled_sobol32* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
@@ -728,8 +784,8 @@ double rocrand_log_normal_double(rocrand_state_scrambled_sobol32* state, double
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_sobol64* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_sobol64* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
@@ -747,8 +803,8 @@ float rocrand_log_normal(rocrand_state_sobol64* state, float mean, float stddev)
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_sobol64 * state, double mean, double stddev)
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_sobol64* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
@@ -766,8 +822,8 @@ double rocrand_log_normal_double(rocrand_state_sobol64 * state, double mean, dou
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_scrambled_sobol64* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_scrambled_sobol64* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
@@ -785,8 +841,8 @@ float rocrand_log_normal(rocrand_state_scrambled_sobol64* state, float mean, flo
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_scrambled_sobol64* state, double mean, double stddev)
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_scrambled_sobol64* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
@@ -804,8 +860,8 @@ double rocrand_log_normal_double(rocrand_state_scrambled_sobol64* state, double
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS
-float rocrand_log_normal(rocrand_state_lfsr113* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float
+    rocrand_log_normal(rocrand_state_lfsr113* state, float mean, float stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
     return expf(mean + (stddev * r));
@@ -825,8 +881,9 @@ float rocrand_log_normal(rocrand_state_lfsr113* state, float mean, float stddev)
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS
-float2 rocrand_log_normal2(rocrand_state_lfsr113* state, float mean, float stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_lfsr113* state,
+                                                               float                  mean,
+                                                               float                  stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -847,8 +904,8 @@ float2 rocrand_log_normal2(rocrand_state_lfsr113* state, float mean, float stdde
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS
-double rocrand_log_normal_double(rocrand_state_lfsr113* state, double mean, double stddev)
+__forceinline__ __device__ __host__ double
+    rocrand_log_normal_double(rocrand_state_lfsr113* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
     return exp(mean + (stddev * r));
@@ -858,7 +915,7 @@ double rocrand_log_normal_double(rocrand_state_lfsr113* state, double mean, doub
  * \brief Returns two log-normally distributed \p double values.
  *
  * Generates and returns two log-normally distributed \p double values using LFSR113
- * generator in \p state, and increments position of the generator by two.
+ * generator in \p state, and increments position of the generator by four.
  * The function uses the Box-Muller transform method to generate two normally distributed
  * values, transforms them to log-normally distributed values, and returns both.
  *
@@ -868,8 +925,9 @@ double rocrand_log_normal_double(rocrand_state_lfsr113* state, double mean, doub
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS
-double2 rocrand_log_normal_double2(rocrand_state_lfsr113* state, double mean, double stddev)
+__forceinline__ __device__ __host__ double2 rocrand_log_normal_double2(rocrand_state_lfsr113* state,
+                                                                       double                 mean,
+                                                                       double stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -884,7 +942,7 @@ double2 rocrand_log_normal_double2(rocrand_state_lfsr113* state, double mean, do
 /**
  * \brief Returns a log-normally distributed \p float value.
  *
- * Generates and returns a log-normally distributed \p float value using ThreeFry
+ * Generates and returns a log-normally distributed \p float value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -893,7 +951,7 @@ double2 rocrand_log_normal_double2(rocrand_state_lfsr113* state, double mean, do
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS float
+__forceinline__ __device__ __host__ float
     rocrand_log_normal(rocrand_state_threefry2x32_20* state, double mean, double stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
@@ -914,21 +972,18 @@ FQUALIFIERS float
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry2x32_20* state,
-                                       float                          mean,
-                                       float                          stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_threefry2x32_20* state,
+                                                               float                          mean,
+                                                               float stddev)
 {
-    auto state1 = rocrand(state);
-    auto state2 = rocrand(state);
-
-    float2 r = rocrand_device::detail::normal_distribution2(state1, state2);
+    float2 r = rocrand_device::detail::normal_distribution2(rocrand2(state));
     return float2{expf(mean + (stddev * r.x)), expf(mean + (stddev * r.y))};
 }
 
 /**
  * \brief Returns a log-normally distributed \p double value.
  *
- * Generates and returns a log-normally distributed \p double value using ThreeFry
+ * Generates and returns a log-normally distributed \p double value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -937,7 +992,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry2x32_20* state,
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS double
+__forceinline__ __device__ __host__ double
     rocrand_log_normal_double(rocrand_state_threefry2x32_20* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
@@ -948,7 +1003,7 @@ FQUALIFIERS double
  * \brief Returns two log-normally distributed \p double values.
  *
  * Generates and returns two log-normally distributed \p double values using Threefry
- * generator in \p state, and increments position of the generator by two.
+ * generator in \p state, and increments position of the generator by four.
  * The function uses the Box-Muller transform method to generate two normally distributed
  * values, transforms them to log-normally distributed values, and returns both.
  *
@@ -958,9 +1013,8 @@ FQUALIFIERS double
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry2x32_20* state,
-                                               double                         mean,
-                                               double                         stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_threefry2x32_20* state, double mean, double stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -975,7 +1029,7 @@ FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry2x32_20* st
 /**
  * \brief Returns a log-normally distributed \p float value.
  *
- * Generates and returns a log-normally distributed \p float value using ThreeFry
+ * Generates and returns a log-normally distributed \p float value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -984,7 +1038,7 @@ FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry2x32_20* st
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS float
+__forceinline__ __device__ __host__ float
     rocrand_log_normal(rocrand_state_threefry2x64_20* state, double mean, double stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
@@ -1005,9 +1059,9 @@ FQUALIFIERS float
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry2x64_20* state,
-                                       float                          mean,
-                                       float                          stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_threefry2x64_20* state,
+                                                               float                          mean,
+                                                               float stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -1019,7 +1073,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry2x64_20* state,
 /**
  * \brief Returns a log-normally distributed \p double value.
  *
- * Generates and returns a log-normally distributed \p double value using ThreeFry
+ * Generates and returns a log-normally distributed \p double value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -1028,7 +1082,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry2x64_20* state,
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS double
+__forceinline__ __device__ __host__ double
     rocrand_log_normal_double(rocrand_state_threefry2x64_20* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
@@ -1049,24 +1103,17 @@ FQUALIFIERS double
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry2x64_20* state,
-                                               double                         mean,
-                                               double                         stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_threefry2x64_20* state, double mean, double stddev)
 {
-    auto state1 = rocrand(state);
-    auto state2 = rocrand(state);
-    auto state3 = rocrand(state);
-    auto state4 = rocrand(state);
-
-    double2 r = rocrand_device::detail::normal_distribution_double2(
-        uint4{state1, state2, state3, state4});
+    double2 r = rocrand_device::detail::normal_distribution_double2(rocrand2(state));
     return double2{exp(mean + (stddev * r.x)), exp(mean + (stddev * r.y))};
 }
 
 /**
  * \brief Returns a log-normally distributed \p float value.
  *
- * Generates and returns a log-normally distributed \p float value using ThreeFry
+ * Generates and returns a log-normally distributed \p float value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -1075,7 +1122,7 @@ FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry2x64_20* st
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS float
+__forceinline__ __device__ __host__ float
     rocrand_log_normal(rocrand_state_threefry4x32_20* state, double mean, double stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
@@ -1085,7 +1132,7 @@ FQUALIFIERS float
 /**
  * \brief Returns two log-normally distributed \p float values.
  *
- * Generates and returns two log-normally distributed \p float values using THREEFRY
+ * Generates and returns two log-normally distributed \p float values using Threefry
  * generator in \p state, and increments position of the generator by two.
  * The function uses the Box-Muller transform method to generate two normally distributed
  * values, transforms them to log-normally distributed values, and returns both.
@@ -1096,9 +1143,9 @@ FQUALIFIERS float
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x32_20* state,
-                                       float                          mean,
-                                       float                          stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_threefry4x32_20* state,
+                                                               float                          mean,
+                                                               float stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -1110,7 +1157,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x32_20* state,
 /**
  * \brief Returns a log-normally distributed \p double value.
  *
- * Generates and returns a log-normally distributed \p double value using ThreeFry
+ * Generates and returns a log-normally distributed \p double value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -1119,7 +1166,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x32_20* state,
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS double
+__forceinline__ __device__ __host__ double
     rocrand_log_normal_double(rocrand_state_threefry4x32_20* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
@@ -1130,7 +1177,7 @@ FQUALIFIERS double
  * \brief Returns two log-normally distributed \p double values.
  *
  * Generates and returns two log-normally distributed \p double values using Threefry
- * generator in \p state, and increments position of the generator by two.
+ * generator in \p state, and increments position of the generator by four.
  * The function uses the Box-Muller transform method to generate two normally distributed
  * values, transforms them to log-normally distributed values, and returns both.
  *
@@ -1140,24 +1187,17 @@ FQUALIFIERS double
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry4x32_20* state,
-                                               double                         mean,
-                                               double                         stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_threefry4x32_20* state, double mean, double stddev)
 {
-    auto state1 = rocrand(state);
-    auto state2 = rocrand(state);
-    auto state3 = rocrand(state);
-    auto state4 = rocrand(state);
-
-    double2 r = rocrand_device::detail::normal_distribution_double2(
-        uint4{state1, state2, state3, state4});
+    double2 r = rocrand_device::detail::normal_distribution_double2(rocrand4(state));
     return double2{exp(mean + (stddev * r.x)), exp(mean + (stddev * r.y))};
 }
 
 /**
  * \brief Returns a log-normally distributed \p float value.
  *
- * Generates and returns a log-normally distributed \p float value using ThreeFry
+ * Generates and returns a log-normally distributed \p float value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -1166,7 +1206,7 @@ FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry4x32_20* st
  *
  * \return Log-normally distributed \p float value
  */
-FQUALIFIERS float
+__forceinline__ __device__ __host__ float
     rocrand_log_normal(rocrand_state_threefry4x64_20* state, double mean, double stddev)
 {
     float r = rocrand_device::detail::normal_distribution(rocrand(state));
@@ -1187,9 +1227,9 @@ FQUALIFIERS float
  *
  * \return Two log-normally distributed \p float value as \p float2
  */
-FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x64_20* state,
-                                       float                          mean,
-                                       float                          stddev)
+__forceinline__ __device__ __host__ float2 rocrand_log_normal2(rocrand_state_threefry4x64_20* state,
+                                                               float                          mean,
+                                                               float stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
@@ -1201,7 +1241,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x64_20* state,
 /**
  * \brief Returns a log-normally distributed \p double value.
  *
- * Generates and returns a log-normally distributed \p double value using ThreeFry
+ * Generates and returns a log-normally distributed \p double value using Threefry
  * generator in \p state, and increments position of the generator by one.
  *
  * \param state  - Pointer to a state to use
@@ -1210,7 +1250,7 @@ FQUALIFIERS float2 rocrand_log_normal2(rocrand_state_threefry4x64_20* state,
  *
  * \return Log-normally distributed \p double value
  */
-FQUALIFIERS double
+__forceinline__ __device__ __host__ double
     rocrand_log_normal_double(rocrand_state_threefry4x64_20* state, double mean, double stddev)
 {
     double r = rocrand_device::detail::normal_distribution_double(rocrand(state));
@@ -1231,17 +1271,13 @@ FQUALIFIERS double
  *
  * \return Two log-normally distributed \p double values as \p double2
  */
-FQUALIFIERS double2 rocrand_log_normal_double2(rocrand_state_threefry4x64_20* state,
-                                               double                         mean,
-                                               double                         stddev)
+__forceinline__ __device__ __host__ double2
+    rocrand_log_normal_double2(rocrand_state_threefry4x64_20* state, double mean, double stddev)
 {
     auto state1 = rocrand(state);
     auto state2 = rocrand(state);
-    auto state3 = rocrand(state);
-    auto state4 = rocrand(state);
 
-    double2 r = rocrand_device::detail::normal_distribution_double2(
-        uint4{state1, state2, state3, state4});
+    double2 r = rocrand_device::detail::normal_distribution_double2(ulonglong2{state1, state2});
     return double2{exp(mean + (stddev * r.x)), exp(mean + (stddev * r.y))};
 }
 
