@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -45,24 +45,22 @@ TEST_P(rocrand_generate_log_normal_tests, float_test)
     float mean = 5.0f;
     float stddev = 2.0f;
     float * data;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&data), size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&data, size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any sizes
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal(generator, data, 1, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal(generator, data, 1, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any alignment
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal(generator, data+1, 2, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal(generator, data + 1, 2, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal(generator, data, size, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal(generator, data, size, mean, stddev));
+    HIP_CHECK(hipDeviceSynchronize());
+
+    // No output pointer
+    ROCRAND_CHECK(rocrand_generate_log_normal(generator, nullptr, size, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     HIP_CHECK(hipFree(data));
@@ -85,24 +83,22 @@ TEST_P(rocrand_generate_log_normal_tests, double_test)
     double mean = 5.0;
     double stddev = 2.0;
     double * data;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&data), size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&data, size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any sizes
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_double(generator, data, 1, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_double(generator, data, 1, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any alignment
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_double(generator, data+1, 2, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_double(generator, data + 1, 2, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_double(generator, data, size, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_double(generator, data, size, mean, stddev));
+    HIP_CHECK(hipDeviceSynchronize());
+
+    // No output pointer
+    ROCRAND_CHECK(rocrand_generate_log_normal_double(generator, nullptr, size, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     HIP_CHECK(hipFree(data));
@@ -125,24 +121,22 @@ TEST_P(rocrand_generate_log_normal_tests, half_test)
     half         mean   = __float2half(5.0f);
     half         stddev = __float2half(2.0f);
     half * data;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&data), size * sizeof(half)));
+    HIP_CHECK(hipMallocHelper(&data, size * sizeof(half)));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any sizes
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_half(generator, data, 1, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_half(generator, data, 1, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     // Any alignment
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_half(generator, data+1, 2, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_half(generator, data + 1, 2, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
-    ROCRAND_CHECK(
-        rocrand_generate_log_normal_half(generator, data, size, mean, stddev)
-    );
+    ROCRAND_CHECK(rocrand_generate_log_normal_half(generator, data, size, mean, stddev));
+    HIP_CHECK(hipDeviceSynchronize());
+
+    // No output pointer
+    ROCRAND_CHECK(rocrand_generate_log_normal_half(generator, nullptr, size, mean, stddev));
     HIP_CHECK(hipDeviceSynchronize());
 
     HIP_CHECK(hipFree(data));
@@ -151,16 +145,26 @@ TEST_P(rocrand_generate_log_normal_tests, half_test)
 
 TEST(rocrand_generate_log_normal_tests, neg_test)
 {
-    const size_t size = 256;
-    float mean = 5.0;
-    float stddev = 2.0;
-    float * data = NULL;
+    const size_t size   = 256;
+    float        mean   = 5.0;
+    float        stddev = 2.0;
+    void*        data   = nullptr;
+
+    rocrand_generator generator = nullptr;
+
+    EXPECT_EQ(rocrand_generate_log_normal(generator, static_cast<float*>(data), size, mean, stddev),
+              ROCRAND_STATUS_NOT_CREATED);
+
+    EXPECT_EQ(rocrand_generate_log_normal_double(generator,
+                                                 static_cast<double*>(data),
+                                                 size,
+                                                 mean,
+                                                 stddev),
+              ROCRAND_STATUS_NOT_CREATED);
 
-    rocrand_generator generator = NULL;
     EXPECT_EQ(
-        rocrand_generate_log_normal(generator, (float *) data, size, mean, stddev),
-        ROCRAND_STATUS_NOT_CREATED
-    );
+        rocrand_generate_log_normal_half(generator, static_cast<half*>(data), size, mean, stddev),
+        ROCRAND_STATUS_NOT_CREATED);
 }
 
 INSTANTIATE_TEST_SUITE_P(rocrand_generate_log_normal_tests,
