@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@
 
 #include <hip/hip_runtime.h>
 
-#define FQUALIFIERS __forceinline__ __host__ __device__
 #include <rocrand/rocrand.h>
 #include <rocrand/rocrand_kernel.h>
 
@@ -63,6 +62,7 @@ __global__ __launch_bounds__(32) void rocrand_kernel(unsigned int* output, const
                        ROCRAND_LFSR113_DEFAULT_SEED_Z,
                        ROCRAND_LFSR113_DEFAULT_SEED_W},
                  subsequence,
+                 123ULL,
                  &state);
 
     unsigned int index = state_id;
@@ -86,6 +86,7 @@ __global__ __launch_bounds__(32) void rocrand_uniform_kernel(float* output, cons
                        ROCRAND_LFSR113_DEFAULT_SEED_Z,
                        ROCRAND_LFSR113_DEFAULT_SEED_W},
                  subsequence,
+                 234ULL,
                  &state);
 
     unsigned int index = state_id;
@@ -110,6 +111,7 @@ __global__ __launch_bounds__(32) void rocrand_uniform_double_kernel(double*
                        ROCRAND_LFSR113_DEFAULT_SEED_Z,
                        ROCRAND_LFSR113_DEFAULT_SEED_W},
                  subsequence,
+                 234ULL,
                  &state);
 
     unsigned int index = state_id;
@@ -128,7 +130,7 @@ __global__ __launch_bounds__(32) void rocrand_normal_kernel(float* output, const
 
     GeneratorState     state;
     const unsigned int subsequence = state_id;
-    rocrand_init(uint4{12345, 67890, 23456, 78901}, subsequence, &state);
+    rocrand_init(uint4{12345, 67890, 23456, 78901}, subsequence, 345ULL, &state);
 
     unsigned int index = state_id;
     while(index < size)
@@ -149,7 +151,7 @@ __global__ __launch_bounds__(32) void rocrand_log_normal_kernel(float* output, c
 
     GeneratorState     state;
     const unsigned int subsequence = state_id;
-    rocrand_init(uint4{12345, 67890, 23456, 78901}, subsequence, &state);
+    rocrand_init(uint4{12345, 67890, 23456, 78901}, subsequence, 456ULL, &state);
 
     unsigned int index = state_id;
     while(index < size)
@@ -172,7 +174,7 @@ __global__ __launch_bounds__(64) void rocrand_poisson_kernel(unsigned int* outpu
 
     GeneratorState     state;
     const unsigned int subsequence = state_id;
-    rocrand_init(uint4{23456, 78901, 34567, 89012}, subsequence, &state);
+    rocrand_init(uint4{23456, 78901, 34567, 89012}, subsequence, 234ULL, &state);
 
     unsigned int index = state_id;
     while(index < size)
@@ -191,7 +193,7 @@ __global__ __launch_bounds__(64) void rocrand_discrete_kernel(
 
     GeneratorState     state;
     const unsigned int subsequence = state_id;
-    rocrand_init(uint4{23456, 78901, 34567, 89012}, subsequence, &state);
+    rocrand_init(uint4{23456, 78901, 34567, 89012}, subsequence, 234ULL, &state);
 
     unsigned int index = state_id;
     while(index < size)
@@ -217,8 +219,7 @@ TEST(rocrand_kernel_lfsr113, rocrand)
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_kernel<state_type>),
@@ -253,7 +254,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_uniform)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -286,7 +287,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_uniform_double)
 
     const size_t output_size = 8192;
     double*      output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -319,7 +320,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_uniform_range)
 
     const size_t output_size = 1 << 26;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -350,7 +351,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_uniform_double_range)
 
     const size_t output_size = 1 << 26;
     double*      output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -381,7 +382,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_normal)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_normal_kernel<state_type>),
@@ -422,7 +423,7 @@ TEST(rocrand_kernel_lfsr113, rocrand_log_normal)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_log_normal_kernel<state_type>),
@@ -472,8 +473,7 @@ TEST_P(rocrand_kernel_lfsr113_poisson, rocrand_poisson)
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
@@ -520,8 +520,7 @@ TEST_P(rocrand_kernel_lfsr113_poisson, rocrand_discrete)
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     rocrand_discrete_distribution discrete_distribution;
