@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@
 
 #include <hip/hip_runtime.h>
 
-#define FQUALIFIERS __forceinline__ __host__ __device__
 #include <rocrand/rocrand.h>
 #include <rocrand/rocrand_kernel.h>
 
@@ -211,8 +210,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand)
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_kernel<state_type>),
@@ -247,7 +245,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_uniform)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -280,7 +278,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_uniform_double)
 
     const size_t output_size = 8192;
     double*      output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -313,7 +311,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_uniform_range)
 
     const size_t output_size = 1 << 26;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -344,7 +342,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_uniform_double_range)
 
     const size_t output_size = 1 << 26;
     double*      output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -375,7 +373,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_normal)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_normal_kernel<state_type>),
@@ -416,7 +414,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_log_normal)
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_log_normal_kernel<state_type>),
@@ -467,8 +465,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_poisson)
 
         const size_t  output_size = 8192;
         unsigned int* output;
-        HIP_CHECK(
-            hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+        HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
         HIP_CHECK(hipDeviceSynchronize());
 
         hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
@@ -518,8 +515,7 @@ TYPED_TEST(rocrand_kernel_mrg, rocrand_discrete)
 
         const size_t  output_size = 8192;
         unsigned int* output;
-        HIP_CHECK(
-            hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+        HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
         HIP_CHECK(hipDeviceSynchronize());
 
         rocrand_discrete_distribution discrete_distribution;
