@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -18,26 +18,25 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include <stdio.h>
 #include <gtest/gtest.h>
+#include <stdio.h>
 
-#include <vector>
 #include <cmath>
+#include <numeric>
 #include <type_traits>
+#include <vector>
 
 #include <hip/hip_runtime.h>
 
-#define FQUALIFIERS __forceinline__ __host__ __device__
 #include <rocrand/rocrand_kernel.h>
 #include <rocrand/rocrand_mtgp32_11213.h>
 
 #include "test_common.hpp"
 #include "test_rocrand_common.hpp"
 
-template <class GeneratorState>
-__global__
-__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
-void rocrand_kernel(GeneratorState * states, unsigned int * output, const size_t size)
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_kernel(
+    GeneratorState* states, unsigned int* output, const size_t size)
 {
     const unsigned int state_id = blockIdx.x;
     unsigned int       index    = blockIdx.x * blockDim.x + threadIdx.x;
@@ -61,10 +60,9 @@ void rocrand_kernel(GeneratorState * states, unsigned int * output, const size_t
     rocrand_mtgp32_block_copy(&state, &states[state_id]);
 }
 
-template <class GeneratorState>
-__global__
-__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
-void rocrand_uniform_kernel(GeneratorState * states, float * output, const size_t size)
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_uniform_kernel(
+    GeneratorState* states, float* output, const size_t size)
 {
     const unsigned int state_id  = blockIdx.x;
     const unsigned int thread_id = threadIdx.x;
@@ -72,7 +70,7 @@ void rocrand_uniform_kernel(GeneratorState * states, float * output, const size_
     unsigned int       stride    = gridDim.x * blockDim.x;
 
     __shared__ GeneratorState state;
-    if (thread_id == 0)
+    if(thread_id == 0)
         state = states[state_id];
     __syncthreads();
 
@@ -88,14 +86,13 @@ void rocrand_uniform_kernel(GeneratorState * states, float * output, const size_
     }
 
     // Save engine with its state
-    if (thread_id == 0)
+    if(thread_id == 0)
         states[state_id] = state;
 }
 
-template <class GeneratorState>
-__global__
-__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
-void rocrand_normal_kernel(GeneratorState * states, float * output, const size_t size)
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_normal_kernel(
+    GeneratorState* states, float* output, const size_t size)
 {
     const unsigned int state_id  = blockIdx.x;
     const unsigned int thread_id = threadIdx.x;
@@ -103,7 +100,7 @@ void rocrand_normal_kernel(GeneratorState * states, float * output, const size_t
     unsigned int       stride    = gridDim.x * blockDim.x;
 
     __shared__ GeneratorState state;
-    if (thread_id == 0)
+    if(thread_id == 0)
         state = states[state_id];
     __syncthreads();
 
@@ -111,9 +108,17 @@ void rocrand_normal_kernel(GeneratorState * states, float * output, const size_t
     const size_t size_rounded_up = r == 0 ? size : size + (blockDim.x - r);
     while(index < size_rounded_up)
     {
-        auto value = rocrand_normal(&state);
         if(index < size)
-            output[index] = value;
+        {
+            if(state_id % 2 == 0)
+            {
+                output[index] = rocrand_normal_double2(&state).x;
+            }
+            else
+            {
+                output[index] = rocrand_normal_double(&state);
+            }
+        }
         // Next position
         index += stride;
     }
@@ -123,10 +128,9 @@ void rocrand_normal_kernel(GeneratorState * states, float * output, const size_t
         states[state_id] = state;
 }
 
-template <class GeneratorState>
-__global__
-__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
-void rocrand_log_normal_kernel(GeneratorState * states, float * output, const size_t size)
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_normal_double_kernel(
+    GeneratorState* states, double* output, const size_t size)
 {
     const unsigned int state_id  = blockIdx.x;
     const unsigned int thread_id = threadIdx.x;
@@ -134,7 +138,7 @@ void rocrand_log_normal_kernel(GeneratorState * states, float * output, const si
     unsigned int       stride    = gridDim.x * blockDim.x;
 
     __shared__ GeneratorState state;
-    if (thread_id == 0)
+    if(thread_id == 0)
         state = states[state_id];
     __syncthreads();
 
@@ -142,22 +146,29 @@ void rocrand_log_normal_kernel(GeneratorState * states, float * output, const si
     const size_t size_rounded_up = r == 0 ? size : size + (blockDim.x - r);
     while(index < size_rounded_up)
     {
-        auto value = rocrand_log_normal(&state, 1.6f, 0.25f);
         if(index < size)
-            output[index] = value;
+        {
+            if(state_id % 2 == 0)
+            {
+                output[index] = rocrand_normal_double2(&state).x;
+            }
+            else
+            {
+                output[index] = rocrand_normal_double(&state);
+            }
+        }
         // Next position
         index += stride;
     }
 
     // Save engine with its state
-    if (thread_id == 0)
+    if(thread_id == 0)
         states[state_id] = state;
 }
 
-template <class GeneratorState>
-__global__
-__launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE)
-void rocrand_poisson_kernel(GeneratorState * states, unsigned int * output, const size_t size, double lambda)
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_log_normal_kernel(
+    GeneratorState* states, float* output, const size_t size)
 {
     const unsigned int state_id  = blockIdx.x;
     const unsigned int thread_id = threadIdx.x;
@@ -165,7 +176,83 @@ void rocrand_poisson_kernel(GeneratorState * states, unsigned int * output, cons
     unsigned int       stride    = gridDim.x * blockDim.x;
 
     __shared__ GeneratorState state;
-    if (thread_id == 0)
+    if(thread_id == 0)
+        state = states[state_id];
+    __syncthreads();
+
+    const size_t r               = size % blockDim.x;
+    const size_t size_rounded_up = r == 0 ? size : size + (blockDim.x - r);
+    while(index < size_rounded_up)
+    {
+        if(index < size)
+        {
+            if(state_id % 2 == 0)
+            {
+                output[index] = rocrand_log_normal2(&state, 1.6f, 0.25f).x;
+            }
+            else
+            {
+                output[index] = rocrand_log_normal(&state, 1.6f, 0.25f);
+            }
+        }
+        // Next position
+        index += stride;
+    }
+
+    // Save engine with its state
+    if(thread_id == 0)
+        states[state_id] = state;
+}
+
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_log_normal_double_kernel(
+    GeneratorState* states, double* output, const size_t size)
+{
+    const unsigned int state_id  = blockIdx.x;
+    const unsigned int thread_id = threadIdx.x;
+    unsigned int       index     = blockIdx.x * blockDim.x + threadIdx.x;
+    unsigned int       stride    = gridDim.x * blockDim.x;
+
+    __shared__ GeneratorState state;
+    if(thread_id == 0)
+        state = states[state_id];
+    __syncthreads();
+
+    const size_t r               = size % blockDim.x;
+    const size_t size_rounded_up = r == 0 ? size : size + (blockDim.x - r);
+    while(index < size_rounded_up)
+    {
+        if(index < size)
+        {
+            if(state_id % 2 == 0)
+            {
+                output[index] = rocrand_log_normal_double2(&state, 1.6f, 0.25f).x;
+            }
+            else
+            {
+                output[index] = rocrand_log_normal_double(&state, 1.6f, 0.25f);
+            }
+        }
+        // Next position
+        index += stride;
+    }
+
+    // Save engine with its state
+    if(thread_id == 0)
+        states[state_id] = state;
+}
+
+template<class GeneratorState>
+__global__ __launch_bounds__(ROCRAND_DEFAULT_MAX_BLOCK_SIZE) void rocrand_poisson_kernel(
+    GeneratorState* states, unsigned int* output, const size_t size, double lambda)
+{
+    const unsigned int state_id  = blockIdx.x;
+    const unsigned int thread_id = threadIdx.x;
+    unsigned int       index     = blockIdx.x * blockDim.x + threadIdx.x;
+    unsigned int       stride    = gridDim.x * blockDim.x;
+
+    __shared__ GeneratorState state;
+    if(thread_id == 0)
         state = states[state_id];
     __syncthreads();
 
@@ -181,7 +268,7 @@ void rocrand_poisson_kernel(GeneratorState * states, unsigned int * output, cons
     }
 
     // Save engine with its state
-    if (thread_id == 0)
+    if(thread_id == 0)
         states[state_id] = state;
 }
 
@@ -198,32 +285,23 @@ TEST(rocrand_kernel_mtgp32, rocrand)
 {
     typedef rocrand_state_mtgp32 state_type;
 
-    state_type * states;
-    hipMallocHelper(reinterpret_cast<void**>(&states), sizeof(state_type) * 8);
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
 
     ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
 
-    const size_t output_size = 8192;
-    unsigned int * output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    const size_t  output_size = 8192;
+    unsigned int* output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
-
-    hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(rocrand_kernel<state_type>),
-        dim3(8), dim3(256), 0, 0,
-        states, output, output_size
-    );
+    rocrand_kernel<state_type><<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
     HIP_CHECK(hipGetLastError());
 
     std::vector<unsigned int> output_host(output_size);
-    HIP_CHECK(
-        hipMemcpy(
-            output_host.data(), output,
-            output_size * sizeof(unsigned int),
-            hipMemcpyDeviceToHost
-        )
-    );
+    HIP_CHECK(hipMemcpy(output_host.data(),
+                        output,
+                        output_size * sizeof(unsigned int),
+                        hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
     HIP_CHECK(hipFree(states));
@@ -241,31 +319,21 @@ TEST(rocrand_kernel_mtgp32, rocrand_uniform)
 {
     typedef rocrand_state_mtgp32 state_type;
 
-    state_type * states;
-    hipMallocHelper(reinterpret_cast<void**>(&states), sizeof(state_type) * 8);
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
 
     ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
 
     const size_t output_size = 8192;
-    float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    float*       output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
-
-    hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
-        dim3(8), dim3(256), 0, 0,
-        states, output, output_size
-    );
+    rocrand_uniform_kernel<state_type><<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
     HIP_CHECK(hipGetLastError());
 
     std::vector<float> output_host(output_size);
     HIP_CHECK(
-        hipMemcpy(
-            output_host.data(), output,
-            output_size * sizeof(float),
-            hipMemcpyDeviceToHost
-        )
-    );
+        hipMemcpy(output_host.data(), output, output_size * sizeof(float), hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
     HIP_CHECK(hipFree(states));
@@ -283,41 +351,62 @@ TEST(rocrand_kernel_mtgp32, rocrand_normal)
 {
     typedef rocrand_state_mtgp32 state_type;
 
-    state_type * states;
-    hipMallocHelper(reinterpret_cast<void**>(&states), sizeof(state_type) * 8);
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
 
     ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
 
     const size_t output_size = 8192;
-    float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    float*       output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
-
-    hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(rocrand_normal_kernel<state_type>),
-        dim3(8), dim3(256), 0, 0,
-        states, output, output_size
-    );
+    rocrand_normal_kernel<state_type><<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
     HIP_CHECK(hipGetLastError());
 
     std::vector<float> output_host(output_size);
     HIP_CHECK(
-        hipMemcpy(
-            output_host.data(), output,
-            output_size * sizeof(float),
-            hipMemcpyDeviceToHost
-        )
-    );
+        hipMemcpy(output_host.data(), output, output_size * sizeof(float), hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
     HIP_CHECK(hipFree(states));
 
-    double mean = 0;
+    double mean = std::accumulate(output_host.begin(), output_host.end(), 0.0) / output_size;
+    EXPECT_NEAR(mean, 0.0, 0.2);
+
+    double stddev = 0;
     for(auto v : output_host)
     {
-        mean += static_cast<double>(v);
+        stddev += std::pow(static_cast<double>(v) - mean, 2);
     }
-    mean = mean / output_size;
+    stddev = stddev / output_size;
+    EXPECT_NEAR(stddev, 1.0, 0.2);
+}
+
+TEST(rocrand_kernel_mtgp32, rocrand_normal_double)
+{
+    typedef rocrand_state_mtgp32 state_type;
+
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
+
+    ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
+
+    const size_t output_size = 8192;
+    double*      output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
+    HIP_CHECK(hipDeviceSynchronize());
+    rocrand_normal_double_kernel<state_type>
+        <<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
+    HIP_CHECK(hipGetLastError());
+
+    std::vector<double> output_host(output_size);
+    HIP_CHECK(
+        hipMemcpy(output_host.data(), output, output_size * sizeof(double), hipMemcpyDeviceToHost));
+    HIP_CHECK(hipDeviceSynchronize());
+    HIP_CHECK(hipFree(output));
+    HIP_CHECK(hipFree(states));
+
+    double mean = std::accumulate(output_host.begin(), output_host.end(), 0.0) / output_size;
     EXPECT_NEAR(mean, 0.0, 0.2);
 
     double stddev = 0;
@@ -333,41 +422,27 @@ TEST(rocrand_kernel_mtgp32, rocrand_log_normal)
 {
     typedef rocrand_state_mtgp32 state_type;
 
-    state_type * states;
-    hipMallocHelper(reinterpret_cast<void**>(&states), sizeof(state_type) * 8);
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
 
     ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
 
     const size_t output_size = 8192;
-    float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    float*       output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
-
-    hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(rocrand_log_normal_kernel<state_type>),
-        dim3(8), dim3(256), 0, 0,
-        states, output, output_size
-    );
+    rocrand_log_normal_kernel<state_type>
+        <<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
     HIP_CHECK(hipGetLastError());
 
     std::vector<float> output_host(output_size);
     HIP_CHECK(
-        hipMemcpy(
-            output_host.data(), output,
-            output_size * sizeof(float),
-            hipMemcpyDeviceToHost
-        )
-    );
+        hipMemcpy(output_host.data(), output, output_size * sizeof(float), hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
     HIP_CHECK(hipFree(states));
 
-    double mean = 0;
-    for(auto v : output_host)
-    {
-        mean += static_cast<double>(v);
-    }
-    mean = mean / output_size;
+    double mean = std::accumulate(output_host.begin(), output_host.end(), 0.0) / output_size;
 
     double stddev = 0;
     for(auto v : output_host)
@@ -377,13 +452,54 @@ TEST(rocrand_kernel_mtgp32, rocrand_log_normal)
     stddev = std::sqrt(stddev / output_size);
 
     double logmean = std::log(mean * mean / std::sqrt(stddev + mean * mean));
-    double logstd = std::sqrt(std::log(1.0f + stddev/(mean * mean)));
+    double logstd  = std::sqrt(std::log(1.0f + stddev / (mean * mean)));
 
     EXPECT_NEAR(1.6, logmean, 1.6 * 0.2);
     EXPECT_NEAR(0.25, logstd, 0.25 * 0.2);
 }
 
-class rocrand_kernel_mtgp32_poisson : public ::testing::TestWithParam<double> { };
+TEST(rocrand_kernel_mtgp32, rocrand_log_normal_double)
+{
+    typedef rocrand_state_mtgp32 state_type;
+
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
+
+    ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
+
+    const size_t output_size = 8192;
+    double*      output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
+    HIP_CHECK(hipDeviceSynchronize());
+    rocrand_log_normal_double_kernel<state_type>
+        <<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size);
+    HIP_CHECK(hipGetLastError());
+
+    std::vector<double> output_host(output_size);
+    HIP_CHECK(
+        hipMemcpy(output_host.data(), output, output_size * sizeof(double), hipMemcpyDeviceToHost));
+    HIP_CHECK(hipDeviceSynchronize());
+    HIP_CHECK(hipFree(output));
+    HIP_CHECK(hipFree(states));
+
+    double mean = std::accumulate(output_host.begin(), output_host.end(), 0.0) / output_size;
+
+    double stddev = 0;
+    for(auto v : output_host)
+    {
+        stddev += std::pow(v - mean, 2);
+    }
+    stddev = std::sqrt(stddev / output_size);
+
+    double logmean = std::log(mean * mean / std::sqrt(stddev + mean * mean));
+    double logstd  = std::sqrt(std::log(1.0f + stddev / (mean * mean)));
+
+    EXPECT_NEAR(1.6, logmean, 1.6 * 0.2);
+    EXPECT_NEAR(0.25, logstd, 0.25 * 0.2);
+}
+
+class rocrand_kernel_mtgp32_poisson : public ::testing::TestWithParam<double>
+{};
 
 TEST_P(rocrand_kernel_mtgp32_poisson, rocrand_poisson)
 {
@@ -391,42 +507,29 @@ TEST_P(rocrand_kernel_mtgp32_poisson, rocrand_poisson)
 
     const double lambda = GetParam();
 
-    state_type * states;
-    hipMallocHelper(reinterpret_cast<void**>(&states), sizeof(state_type) * 8);
+    state_type* states;
+    hipMallocHelper(&states, sizeof(state_type) * 8);
 
     ROCRAND_CHECK(rocrand_make_state_mtgp32(states, mtgp32dc_params_fast_11213, 8, 0));
 
-    const size_t output_size = 8192;
-    unsigned int * output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    const size_t  output_size = 8192;
+    unsigned int* output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
-
-    hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
-        dim3(8), dim3(256), 0, 0,
-        states, output, output_size, lambda
-    );
+    rocrand_poisson_kernel<state_type>
+        <<<dim3(8), dim3(256), 0, 0>>>(states, output, output_size, lambda);
     HIP_CHECK(hipGetLastError());
 
     std::vector<unsigned int> output_host(output_size);
-    HIP_CHECK(
-        hipMemcpy(
-            output_host.data(), output,
-            output_size * sizeof(unsigned int),
-            hipMemcpyDeviceToHost
-        )
-    );
+    HIP_CHECK(hipMemcpy(output_host.data(),
+                        output,
+                        output_size * sizeof(unsigned int),
+                        hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
     HIP_CHECK(hipFree(states));
 
-    double mean = 0;
-    for(auto v : output_host)
-    {
-        mean += static_cast<double>(v);
-    }
-    mean = mean / output_size;
+    double mean = std::accumulate(output_host.begin(), output_host.end(), 0.0) / output_size;
 
     double variance = 0;
     for(auto v : output_host)
@@ -439,8 +542,8 @@ TEST_P(rocrand_kernel_mtgp32_poisson, rocrand_poisson)
     EXPECT_NEAR(variance, lambda, std::max(1.0, lambda * 1e-1));
 }
 
-const double lambdas[] = { 1.0, 5.5, 20.0, 100.0, 1234.5, 5000.0 };
+const double lambdas[] = {1.0, 5.5, 20.0, 100.0, 1234.5, 5000.0};
 
 INSTANTIATE_TEST_SUITE_P(rocrand_kernel_mtgp32_poisson,
-                        rocrand_kernel_mtgp32_poisson,
-                        ::testing::ValuesIn(lambdas));
+                         rocrand_kernel_mtgp32_poisson,
+                         ::testing::ValuesIn(lambdas));
