@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@
 
 #include <hip/hip_runtime.h>
 
-#define FQUALIFIERS __forceinline__ __host__ __device__
 #include <rocrand/rocrand_kernel.h>
 #include <rocrand/rocrand_sobol32_precomputed.h>
 
@@ -156,16 +155,17 @@ TEST(rocrand_kernel_sobol32, rocrand)
 
     const size_t output_size = 8192;
     unsigned int * output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
+    const unsigned int* h_directions;
+    ROCRAND_CHECK(
+        rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
     unsigned int * m_vector;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&m_vector), sizeof(unsigned int) * 8 * 32));
-    HIP_CHECK(hipMemcpy(m_vector,
-                        rocrand_h_sobol32_direction_vectors,
-                        sizeof(unsigned int) * 8 * 32,
-                        hipMemcpyHostToDevice));
+    HIP_CHECK(hipMallocHelper(&m_vector, sizeof(unsigned int) * 8 * 32));
+    HIP_CHECK(
+        hipMemcpy(m_vector, h_directions, sizeof(unsigned int) * 8 * 32, hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(
@@ -202,15 +202,17 @@ TEST(rocrand_kernel_sobol32, rocrand_uniform)
 
     const size_t output_size = 8192;
     float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
+    const unsigned int* h_directions;
+    ROCRAND_CHECK(
+        rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
     unsigned int * m_vector;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&m_vector), sizeof(unsigned int) * 8 * 32));
-    HIP_CHECK(hipMemcpy(m_vector,
-                        rocrand_h_sobol32_direction_vectors,
-                        sizeof(unsigned int) * 8 * 32,
-                        hipMemcpyHostToDevice));
+    HIP_CHECK(hipMallocHelper(&m_vector, sizeof(unsigned int) * 8 * 32));
+    HIP_CHECK(
+        hipMemcpy(m_vector, h_directions, sizeof(unsigned int) * 8 * 32, hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(
@@ -247,15 +249,17 @@ TEST(rocrand_kernel_sobol32, rocrand_normal)
 
     const size_t output_size = 8192;
     float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
+    const unsigned int* h_directions;
+    ROCRAND_CHECK(
+        rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
     unsigned int * m_vector;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&m_vector), sizeof(unsigned int) * 8 * 32));
-    HIP_CHECK(hipMemcpy(m_vector,
-                        rocrand_h_sobol32_direction_vectors,
-                        sizeof(unsigned int) * 8 * 32,
-                        hipMemcpyHostToDevice));
+    HIP_CHECK(hipMallocHelper(&m_vector, sizeof(unsigned int) * 8 * 32));
+    HIP_CHECK(
+        hipMemcpy(m_vector, h_directions, sizeof(unsigned int) * 8 * 32, hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(
@@ -300,15 +304,17 @@ TEST(rocrand_kernel_sobol32, rocrand_log_normal)
 
     const size_t output_size = 8192;
     float * output;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
+    const unsigned int* h_directions;
+    ROCRAND_CHECK(
+        rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
     unsigned int * m_vector;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&m_vector), sizeof(unsigned int) * 8 * 32));
-    HIP_CHECK(hipMemcpy(m_vector,
-                        rocrand_h_sobol32_direction_vectors,
-                        sizeof(unsigned int) * 8 * 32,
-                        hipMemcpyHostToDevice));
+    HIP_CHECK(hipMallocHelper(&m_vector, sizeof(unsigned int) * 8 * 32));
+    HIP_CHECK(
+        hipMemcpy(m_vector, h_directions, sizeof(unsigned int) * 8 * 32, hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(
@@ -359,18 +365,19 @@ TEST_P(rocrand_kernel_sobol32_poisson, rocrand_poisson)
 
     const double lambda = GetParam();
 
+    const unsigned int* h_directions;
+    ROCRAND_CHECK(
+        rocrand_get_direction_vectors32(&h_directions, ROCRAND_DIRECTION_VECTORS_32_JOEKUO6));
+
     unsigned int * m_vector;
-    HIP_CHECK(hipMallocHelper(reinterpret_cast<void**>(&m_vector), sizeof(unsigned int) * 8 * 32));
-    HIP_CHECK(hipMemcpy(m_vector,
-                        rocrand_h_sobol32_direction_vectors,
-                        sizeof(unsigned int) * 8 * 32,
-                        hipMemcpyHostToDevice));
+    HIP_CHECK(hipMallocHelper(&m_vector, sizeof(unsigned int) * 8 * 32));
+    HIP_CHECK(
+        hipMemcpy(m_vector, h_directions, sizeof(unsigned int) * 8 * 32, hipMemcpyHostToDevice));
     HIP_CHECK(hipDeviceSynchronize());
 
     const size_t output_size = 8192;
     unsigned int * output;
-    HIP_CHECK(
-        hipMallocHelper(reinterpret_cast<void**>(&output), output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(
