@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@
 
 #include <hip/hip_runtime.h>
 
-#define FQUALIFIERS __forceinline__ __host__ __device__
 #include <rocrand/rocrand.h>
 #include <rocrand/rocrand_kernel.h>
 
@@ -51,7 +50,7 @@ __global__ __launch_bounds__(32) void rocrand_init_kernel(GeneratorState*    sta
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_kernel(unsigned int* output, const size_t size)
+__global__ __launch_bounds__(32) void rocrand_kernel(unsigned long long* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -186,23 +185,23 @@ __global__ __launch_bounds__(32) void rocrand_discrete_kernel(
     }
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_state_threefry_type)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_state_threefry_type)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
-    EXPECT_EQ(alignof(state_type), alignof(uint4));
+    typedef rocrand_state_threefry4x64_20 state_type;
+    EXPECT_EQ(alignof(state_type), alignof(ulonglong4));
     EXPECT_EQ(sizeof(state_type[32]), 32 * sizeof(state_type));
-    // TODO: Enable once uint4 trivially copyable.
+    // TODO: Enable once ulonglong4 trivially copyable.
     //EXPECT_TRUE(std::is_trivially_copyable<state_type>::value);
     EXPECT_TRUE(std::is_trivially_destructible<state_type>::value);
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand)
+TEST(rocrand_kernel_threefry4x64_20, rocrand)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
-    const size_t  output_size = 8192;
-    unsigned int* output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(unsigned int)));
+    const size_t        output_size = 8192;
+    unsigned long long* output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned long long)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_kernel<state_type>),
@@ -214,10 +213,10 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand)
                        output_size);
     HIP_CHECK(hipGetLastError());
 
-    std::vector<unsigned int> output_host(output_size);
+    std::vector<unsigned long long> output_host(output_size);
     HIP_CHECK(hipMemcpy(output_host.data(),
                         output,
-                        output_size * sizeof(unsigned int),
+                        output_size * sizeof(unsigned long long),
                         hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
@@ -225,19 +224,19 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand)
     double mean = 0;
     for(auto v : output_host)
     {
-        mean += static_cast<double>(v) / UINT_MAX;
+        mean += static_cast<double>(v) / static_cast<double>(ULLONG_MAX);
     }
     mean = mean / output_size;
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_uniform)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -264,13 +263,13 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_double)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_uniform_double)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 8192;
     double*      output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -297,13 +296,13 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_double)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_range)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_uniform_range)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 1 << 26;
     float*       output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_kernel<state_type>),
@@ -328,13 +327,13 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_range)
     }
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_double_range)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_uniform_double_range)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 1 << 26;
     double*      output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(double)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(double)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_uniform_double_kernel<state_type>),
@@ -359,13 +358,13 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_uniform_double_range)
     }
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_normal)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_normal)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_normal_kernel<state_type>),
@@ -400,13 +399,13 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_normal)
     EXPECT_NEAR(stddev, 1.0, 0.2);
 }
 
-TEST(rocrand_kernel_threefry4x32_20, rocrand_log_normal)
+TEST(rocrand_kernel_threefry4x64_20, rocrand_log_normal)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(float)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(float)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_log_normal_kernel<state_type>),
@@ -445,18 +444,18 @@ TEST(rocrand_kernel_threefry4x32_20, rocrand_log_normal)
     EXPECT_NEAR(0.25, logstd, 0.25 * 0.2);
 }
 
-class rocrand_kernel_threefry4x32_20_poisson : public ::testing::TestWithParam<double>
+class rocrand_kernel_threefry4x64_20_poisson : public ::testing::TestWithParam<double>
 {};
 
-TEST_P(rocrand_kernel_threefry4x32_20_poisson, rocrand_poisson)
+TEST_P(rocrand_kernel_threefry4x64_20_poisson, rocrand_poisson)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const double lambda = GetParam();
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
@@ -495,15 +494,15 @@ TEST_P(rocrand_kernel_threefry4x32_20_poisson, rocrand_poisson)
     EXPECT_NEAR(variance, lambda, std::max(1.0, lambda * 1e-1));
 }
 
-TEST_P(rocrand_kernel_threefry4x32_20_poisson, rocrand_discrete)
+TEST_P(rocrand_kernel_threefry4x64_20_poisson, rocrand_discrete)
 {
-    typedef rocrand_state_threefry4x32_20 state_type;
+    typedef rocrand_state_threefry4x64_20 state_type;
 
     const double lambda = GetParam();
 
     const size_t  output_size = 8192;
     unsigned int* output;
-    HIP_CHECK(hipMallocHelper((void**)&output, output_size * sizeof(unsigned int)));
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     rocrand_discrete_distribution discrete_distribution;
@@ -548,6 +547,6 @@ TEST_P(rocrand_kernel_threefry4x32_20_poisson, rocrand_discrete)
 
 const double lambdas[] = {1.0, 5.5, 20.0, 100.0, 1234.5, 5000.0};
 
-INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry4x32_20_poisson,
-                         rocrand_kernel_threefry4x32_20_poisson,
+INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry4x64_20_poisson,
+                         rocrand_kernel_threefry4x64_20_poisson,
                          ::testing::ValuesIn(lambdas));
